use crate::queue::Message;
use std::{
    sync::mpsc::{channel, Receiver, Sender},
    thread::spawn,
};

struct BackEnd {
    tx: Sender<Message>,
    rx: Receiver<Message>,
}

impl BackEnd {
    fn new(tx: Sender<Message>, rx: Receiver<Message>) -> Self {
        Self {
            tx: tx,
            rx: rx,
        }
    }

    fn start(queue_tx: Sender<Message>) {
        spawn(move || {
            let (tx, rx) = channel();
            let service = Self::new(queue_tx.clone(), rx);
            queue_tx.send(tx.clone().into()).unwrap();
            service.listen();
        });
    }

    fn listen(&self) {
        loop {
            self.rx.recv().unwrap();
        }
    }
}

#[cfg(test)]
mod backends {
    use std::time::Duration;
    use super::*;

    #[test]
    fn get_registered() {
        let (tx, rx) = channel();
        BackEnd::start(tx);
        match rx.recv_timeout(Duration::from_millis(500)).unwrap() {
            Message::Register(_) => {},
            _ => unreachable!("should register the service"),
        }
    }
}
