use crate::{queue::Message, session2::SessionMessage, Field, Request};
use std::{
    sync::mpsc::{channel, Receiver, Sender},
    thread::spawn,
};

struct FrontEnd {
    tx: Sender<Message>,
    rx: Receiver<Message>,
}

impl FrontEnd {
    fn new(tx: Sender<Message>, rx: Receiver<Message>) -> Self {
        Self { tx: tx, rx: rx }
    }

    fn start(queue_tx: Sender<Message>) {
        spawn(move || {
            let (tx, rx) = channel();
            let frontend = FrontEnd::new(queue_tx.clone(), rx);
            queue_tx.send(tx.into()).unwrap();
            frontend.listen();
        });
    }

    fn listen(&self) {
        loop {
            self.rx.recv().unwrap();
            let id: Option<Field> = None;
            let msg: SessionMessage = id.into();
            self.tx.send(msg.into()).unwrap();
        }
    }
}

#[cfg(test)]
mod frontends {
    use super::*;
    use crate::{
        create_request::empty_request, session2::create_session_message::create_session_new,
    };
    use std::time::Duration;

    fn run_service() -> (Sender<Message>, Receiver<Message>) {
        let (tx, rx) = channel();
        let service_tx: Sender<Message>;
        FrontEnd::start(tx);
        match rx.recv().unwrap() {
            Message::Register(result) => service_tx = result,
            _ => unreachable!("should register the service"),
        }
        (service_tx, rx)
    }

    #[test]
    fn gets_registered() {
        let (tx, rx) = channel();
        FrontEnd::start(tx);
        match rx.recv().unwrap() {
            Message::Register(_) => {}
            _ => unreachable!("should register the service"),
        }
    }

    #[test]
    fn handle_request() {
        let (tx, rx) = run_service();
        let (req, _) = empty_request();
        tx.send(req.into()).unwrap();
        match rx.recv_timeout(Duration::from_millis(500)).unwrap() {
            Message::SessMsg(output) => match output {
                SessionMessage::Validate(result) => assert!(result.is_none()),
                _ => unreachable!("Should have sent a val9idate"),
            },
            _ => unreachable!("Should have generated a session message"),
        }
        let (new_sess, sess_id) = create_session_new();
        tx.send(new_sess.into()).unwrap();
        // generate backend request.
    }
}
